/**
* ap.h - P2PWNC access point module
*
* Copyright (C) 2004-2005 Pantelis Frangoudis, Elias Efstathiou
*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*
* In addition, as a special exception, the copyright holders
* give permission to link the code of this program with the
* OpenSSL toolkit (or with modified versions of OpenSSL that use the
* same license as OpenSSL, version 0.9.8a), and distribute linked
* combinations including the two.  You must obey the GNU General Public License
* in all respects for all of the code used other than OpenSSL.  If you modify
* this file, you may extend this exception to your version of the
* file, but you are not obligated to do so.  If you do not wish to
* do so, delete this exception statement from your version.
*/

#ifndef _PWNC_AP_H
#define _PWNC_AP_H

#define TCA_SUPPORT 1

#include "io.h"
#include "crypt.h"
#include "conffile.h"
#include "threads.h"

#include <signal.h>
#include <errno.h>

//################################################################
typedef struct __pwnc_client {
	unsigned long client_ip;
	unsigned long timestamp;
	int sockfd;
	PWNC_CERT *client_cert;
	PWNC_RCPT *rcpt;
} PWNC_CLIENT;


#define SAFE_FREE_CLIENT(x) \
	SAFE_FREE_CERT(x->client_cert); \
	if (x->rcpt) { \
		SAFE_FREE_RCPT(x->rcpt); \
	} \
	SAFE_FREE(x); \


#define DECLARE_LHASH_HASH_FN(f_name,o_type) \
         unsigned long f_name##_LHASH_HASH(const void *);
 #define IMPLEMENT_LHASH_HASH_FN(f_name,o_type) \
         unsigned long f_name##_LHASH_HASH(const void *arg) { \
                 o_type a = (o_type)arg; \
                 return f_name(a); }
 #define LHASH_HASH_FN(f_name) f_name##_LHASH_HASH

 #define DECLARE_LHASH_COMP_FN(f_name,o_type) \
         int f_name##_LHASH_COMP(const void *, const void *);
 #define IMPLEMENT_LHASH_COMP_FN(f_name,o_type) \
         int f_name##_LHASH_COMP(const void *arg1, const void *arg2) { \
                 o_type a = (o_type)arg1; \
                 o_type b = (o_type)arg2; \
                 return f_name(a,b); }
 #define LHASH_COMP_FN(f_name) f_name##_LHASH_COMP

unsigned long pwnc_client_hash(const PWNC_CLIENT *to_hash);
int pwnc_client_cmp(const PWNC_CLIENT *arg1, const PWNC_CLIENT *arg2);
static IMPLEMENT_LHASH_HASH_FN(pwnc_client_hash, const PWNC_CLIENT *);
static IMPLEMENT_LHASH_COMP_FN(pwnc_client_cmp, const PWNC_CLIENT *);
//################################################################

//################################################################
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/fcntl.h>
#define ACTIVATE   0xFEED0000
#define DEACTIVATE   0xFEED0001
#define GET_STATS   0xFEED0002
#define REMOVE_REC 0xFEED0003
#define ADD_REC 0xFEED0004

#define MAX_CLIENTS 1024

struct stats {
	unsigned long int nClientIP;
	unsigned int nTotalPacketsIn;
	unsigned int nTotalPacketsOut;
	unsigned int nTotalBytesIn;
	unsigned int nTotalBytesOut;
};

struct client_list {
	unsigned int nRecNum;
	struct stats clist[MAX_CLIENTS];
};
//################################################################


//################################################################
//configuration
unsigned char str_tca_ipaddr[20];
unsigned char str_pub_key_path[80];
unsigned char str_priv_key_path[80];
unsigned char str_conf_file_path[80];
unsigned char str_iptables_path[80];
unsigned char str_subnet[20];
unsigned char str_mask[4];
int RREQ_DELAY_SEC = 10; //sends an rreq to the client every 10 seconds (default)
int RREQ_DELAY_USEC = 0;
int patient = 0; //not patient by default
int max_threads = 50; //default: 50 threads maximum

int server_port;
int mq_key;
#define DEF_CONF_PATH "../conf/p2pwnc_ap.conf"
#define DEF_CONF_FNAME "p2pwnc_ap.conf"
//################################################################


//################################################################
//mqueue specific decls
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/msg.h>
//key_t mq_key = 100;
int mq_id;
#define MTYPE_FW_CMD 10 //message type: itpables command (the only valid mtype)
#define MTYPE_FW_EXIT 20 //server exits...
typedef struct msgbuf {
	long mtype;
	char mtext[1];
} MSG_BUF;
//################################################################


//################################################################
//hash table storing client state
static LHASH *clients_ht;

pthread_mutex_t ht_mutex;
//pthread_t pool[50];

PWNC_PUBKEY *server_pub;

struct sockaddr_in server_addr;
int sockfd;

const char *tca;
//################################################################

void daemonize();

#endif /* _PWNC_AP_H */
